module AI where

import Control.Concurrent.MVar
import State

data RobberAI state_t = RobberAI {
    rai_name :: String,
    rai_init :: InitAI state_t,
    rai_move :: StepAI state_t WorldUpdate PlayerState
}

data CopAI state_t = CopAI {
    cai_name :: String,
    cai_init_type :: PlayerType,
    cai_init :: InitAI state_t,
    cai_inform :: StepAI state_t WorldUpdate [Inform],
    cai_plan :: StepAI state_t [Inform] [Plan],
    cai_vote :: StepAI state_t [Plan] [PlayerName],
    cai_move :: StepAI state_t Winner PlayerState
}

---------------
-- COMPUTATIONS
---------------
--
-- computations run in threads and are killed after some time limit.
-- they communicate via MVars: the current state and the computation
-- result are read from a "result" MVar after the thread is killed.
-- the InitAI call is made once, and relevant StepAI calls are
-- repeated as the gameplay requires.
-- 
-- the InitAI call receives an empty state MVar, and each subsequent
-- StepAI call receives the state computed by from the last call.
-- 
-- each StepAI call receives an empty result MVar, which will be read after
-- the thread is killed.  if, at that point, the result MVar is empty,
-- the caller terminates with an error, so it is wise to seed the result var
-- early with a legal result.

type InitAI state_t = WorldSkeleton -> MVar state_t -> IO ()
type StepAI state_t update_t result_t =
    state_t -> update_t -> MVar (state_t, result_t) -> IO ()

