module CopAimless where

import Control.Concurrent.MVar
import ConcurrentUtil
import State
import AI
import LegalMoves
import RandomUtils

-- cop-statue moves randomly, never shares information, never submits
-- plans, and votes a random permutation of cop names

cop_aimless = CopAI { cai_name = "cop-aimless",
                       cai_init_type = PT_cop_foot,
                       cai_init = cop_aimless_init,
                       cai_inform = cop_aimless_no_op,
                       cai_plan = cop_aimless_no_op,
                       cai_vote = cop_aimless_vote,
                       cai_move = cop_aimless_move }

data AimlessState = AimlessState {
    cs_skeleton :: WorldSkeleton,
    cs_current_ps :: PlayerState
}

cop_aimless_init :: InitAI AimlessState
cop_aimless_init skel state_var =
    putMVar state_var (AimlessState {
                           cs_skeleton = skel,
                           cs_current_ps = PlayerState {
                                               ps_type = PT_cop_foot,
                                               ps_loc = ws_hq skel }})

cop_aimless_no_op :: StepAI AimlessState update_t [result_elt_t]
cop_aimless_no_op state update result_var =
    putMVar result_var (state, [])

cop_aimless_vote :: StepAI AimlessState [Plan] Vote
cop_aimless_vote state plans result_var =
    do vote <- random_list_perm (ws_cops (cs_skeleton state))
       putMVarF2 result_var (state, vote)

cop_aimless_move :: StepAI AimlessState Winner PlayerState
cop_aimless_move state winner result_var =
    do -- seed with identity move in case time runs out
       putMVarF2 result_var (state, cs_current_ps state)
       let legal_states = legalMoves (cs_skeleton state) (cs_current_ps state)
       next_ps <- random_list_elt legal_states
       let next_state = state { cs_current_ps = next_ps }
       swapMVarF2_ result_var (next_state, next_ps)

