module CopStatue where

import Control.Concurrent.MVar
import State
import AI

-- cop-statue never moves from HQ, never shares information, never submits
-- plans, and always votes the same way, in the order sent by the server at
-- initialization time

cop_statue = CopAI { cai_name = "cop-statue",
                     cai_init_type = PT_cop_foot,
                     cai_init = cop_statue_init,
                     cai_inform = cop_statue_no_op,
                     cai_plan = cop_statue_no_op,
                     cai_vote = cop_statue_vote,
                     cai_move = cop_statue_move }

cop_statue_init :: InitAI (PlayerState, [PlayerName])
cop_statue_init skl state_var =
    putMVar state_var (PlayerState { ps_type = PT_cop_foot,
                                     ps_loc = ws_hq skl },
                       ws_cops skl)

cop_statue_no_op :: StepAI (PlayerState, [PlayerName]) update_t [result_elt_t]
cop_statue_no_op state update result_var =
    putMVar result_var (state, [])

cop_statue_vote :: StepAI (PlayerState, [PlayerName]) update_t [PlayerName]
cop_statue_vote state update result_var =
    putMVar result_var (state, snd state)

cop_statue_move :: StepAI (PlayerState, [PlayerName]) update_t PlayerState
cop_statue_move state update result_var =
    putMVar result_var (state, fst state)

