module Evaluate where

import SearchState
import SearchMove
import State
import Data.Maybe
import RobberTracking
import Data.List

newtype EvalScore = EvalScore [Int]
    deriving (Eq, Show)

instance Ord EvalScore where
    compare (EvalScore is) (EvalScore js) = compareEvalScore is js

compareEvalScore [] _ = EQ
compareEvalScore _ [] = EQ
compareEvalScore (i:is) (j:js) =
    let curOrdering = compare i j in
        case curOrdering
        of
        EQ -> compareEvalScore is js
        _ -> curOrdering

evaluateGameOver :: WorldSkeleton -> SearchState -> Maybe EvalScore
evaluateGameOver ws ss
    | isJust (ss_fake_robber_states ss) =
        Nothing
    | isRobberCaught ws ss =
        Just (EvalScore [1, 0])
    | isTimeUp ss =
        Just (EvalScore [-1, 0])
    | otherwise =
        Nothing

evaluate :: WorldSkeleton -> SearchState -> EvalScore
evaluate ws ss
    | ssRobberToMove ss = error "Can only evaluate with cop to move"
    | otherwise =
        let (impossibleLocs:_) = ss_impossible_list ss
            copStates = ss_cop_states ss
            robberLoc = stateToLoc (ss_robber_state ss)
            copDistances =
                map (snd . ((flip (ws_plot_course ws)) robberLoc)) copStates
            sortedCopDistances = sort copDistances in
        EvalScore ([0, lsSize impossibleLocs] ++
                   map negate sortedCopDistances)

isRobberCaught ws ss =
    let robberLoc = stateToLoc (ss_robber_state ss)
        copLocs = map stateToLoc (ss_cop_states ss) in
        elem robberLoc copLocs

isTimeUp ss =
    ss_epoch ss == 200

nextDepth ws ss 0
    | isJust (ss_fake_robber_states ss) = 3
    | ssRobberToMove ss = 1
    | ssCopToMove ss = 2
nextDepth _ _ i = i + 2
