module PlotCourse where

import Data.Graph.Inductive
import State
import qualified Data.Map as M
import Data.Maybe

plot_course :: WorldSkeleton -> PlayerState -> LocName
            -> ([PlayerState], Distance)
plot_course ws = (ws_plot_course ws)

plot_course_impl :: WorldSkeleton -> PlayerState -> LocName
                 -> ([PlayerState], Distance)
plot_course_impl ws = plot_course_with_map ws (ws_map_ps ws)

plot_course_with_map :: WorldSkeleton -> PlayerStateMap -> PlayerState
                     -> LocName -> ([PlayerState], Int)
plot_course_with_map ws map_ps start_ps end_loc =
    let start_node = (ws_ps_to_node ws) start_ps
        end_nodes | ps_type start_ps == PT_robber =
             [(ws_ps_to_node ws)
              (PlayerState { ps_loc = end_loc, ps_type = PT_robber })]
                  | otherwise =
            map (ws_ps_to_node ws)
                [PlayerState { ps_loc = end_loc, ps_type = PT_cop_foot },
                 PlayerState { ps_loc = end_loc, ps_type = PT_cop_car }]
        (path, cost) =
               case [(p, path_cost map_ps p) -- XXX assuming reachable
                     | end <- end_nodes,
                       let p = sp start_node end map_ps] of
               [(path, d)] -> (path,d)
               [(p1, d1), (p2, d2)] -> if d1 < d2 then (p1,d1) else (p2,d2)
    in  (map (ws_node_to_ps ws) path, cost)

path_cost g [] = 0
path_cost g [start] =
    foldr1 min [cst | (_ , end, cst) <- out g start, end == start]
path_cost g (start:rest) = cost_to g start rest

cost_to g start [] = 0
cost_to g start (end:rest) =
  let cost = foldr1 min [cst | (_, end', cst) <- out g start, end' == end]
  in  cost + cost_to g end rest

compute_ws_plot_course ws =
    let allLocs = ws_all_robber_loc_names ws
        allKeys =
            concat
            [[(PlayerState { ps_loc = fromLoc, ps_type = PT_cop_foot}, toLoc),
              (PlayerState { ps_loc = fromLoc, ps_type = PT_cop_car}, toLoc),
              (PlayerState { ps_loc = fromLoc, ps_type = PT_robber}, toLoc)]
             |
             fromLoc <- allLocs,
             toLoc <- allLocs]
        associations =
            [(key, plot_course_impl ws start_ps end_loc)
             |
             key@(start_ps, end_loc) <- allKeys]
        theMap = M.fromList associations in
        \ start_ps end_loc -> fromJust (M.lookup (start_ps, end_loc) theMap)
