
module RobberFearless where

import List
import Control.Concurrent.MVar

import ConcurrentUtil
import State
import AI
import LegalMoves
import RandomUtils
import PlotCourse

-- robber-fearless goes for the richest bank

robber_fearless = RobberAI { rai_name = "robber-fearless",
                             rai_init = robber_fearless_init,
                             rai_move = robber_fearless_move }

data FearlessState = FearlessState {
    fs_ws :: WorldSkeleton,
    fs_current_ps :: PlayerState
}

robber_fearless_init :: InitAI FearlessState
robber_fearless_init skel state_var =
    putMVar state_var
        (FearlessState { fs_ws = skel,
                         fs_current_ps = PlayerState {
                                             ps_type = PT_robber,
                                             ps_loc = ws_robber_start skel }})

robber_fearless_move :: StepAI FearlessState WorldUpdate PlayerState
robber_fearless_move state update result_var =
    do -- seed with identity move in case time runs out
       let here = fs_current_ps state
       putMVarF2 result_var (state, here)
       let ((_, _, _, bank_path):_) =
               sort [(-val, cost, loc, path)
                     | (loc,val) <- wu_bank_values update,
                       let (path,cost) = plot_course (fs_ws state) here loc]
           move = case bank_path of
                  (here : next : _) -> next
                  [here] -> here
                  [] -> error "robber_fearless_move: phantom bank"
           next_state = state { fs_current_ps = move }
       swapMVarF2_ result_var (next_state, move)

