module SearchMove where

import State
import LegalMoves
import Data.Maybe
import SearchState
import RobberTracking
import RobberTrackingImpl

data SearchMove =
    CopMove [PlayerState] | RobberMove PlayerState
    deriving (Show)

ssAdvance :: WorldSkeleton -> SearchState -> SearchMove -> SearchState
ssAdvance ws ss (RobberMove robberState)
    | isJust (ss_fake_robber_states ss) =
        ss { ss_robber_state = robberState,
             ss_fake_robber_states = Nothing }
    | ssRobberToMove ss =
        let ss' =
              ss { ss_epoch = ss_epoch ss + 1,
                   ss_robber_state = robberState } in
            ss' { ss_impossible_list =
                      nextImpossibleList
                      ws
                      (ss_impossible_list ss')
                      (ssCurImpossibleLocs ws ss') }

ssAdvance ws ss (CopMove copStates)
    | ssCopToMove ss =
        ss { ss_epoch = ss_epoch ss + 1,
             ss_cop_states = copStates }

ssRobberToMove ss = robberToMove (ss_epoch ss)
ssCopToMove ss = copToMove (ss_epoch ss)

ssInitialMoves :: WorldSkeleton -> SearchState -> [SearchMove]
ssInitialMoves ws ss
    | isJust (ss_fake_robber_states ss) =
        map RobberMove (fromJust (ss_fake_robber_states ss))
    | ssRobberToMove ss =
        map RobberMove (legalMoves ws (ss_robber_state ss))
    | ssCopToMove ss =
        [CopMove (ss_cop_states ss)]

ssRefineMoves :: WorldSkeleton -> SearchState -> SearchMove ->  [SearchMove]
ssRefineMoves ws ss move
    | isJust (ss_fake_robber_states ss) = []
    | ssRobberToMove ss = []
    | ssCopToMove ss =
        let CopMove moveStates = move
            len = length moveStates in
        concatMap (copRefineMovesHelper ws ss moveStates) [0..(len-1)]

copRefineMovesHelper ws ss moveStates idx =
    let copStates = ss_cop_states ss
        copState = copStates !! idx
        moveState = moveStates !! idx
        newMoves = [CopMove (listSubst moveStates idx newState) |
                     newState <- legalMoves ws copState,
                     newState /= moveState] in
        newMoves

listSubst l idx newVal =
    let (left, (_:right)) = splitAt idx l in
        left ++ [newVal] ++ right

ssLegalMoves :: WorldSkeleton -> SearchState -> [SearchMove]
ssLegalMoves ws ss
    | isJust (ss_fake_robber_states ss) =
        map RobberMove (fromJust (ss_fake_robber_states ss))
    | ssRobberToMove ss =
        map RobberMove (legalMoves ws (ss_robber_state ss))
    | ssCopToMove ss =
        map CopMove (copLegalMovesHelper ws (ss_cop_states ss))

copLegalMovesHelper ws [] = [[]]
copLegalMovesHelper ws (copState:copStates) =
    [(move:moves)
     |
     move <- legalMoves ws copState,
     moves <- copLegalMovesHelper ws copStates]
