module Main where

import System
import Text.ParserCombinators.Parsec

import State
import Parser
import Protocol

main :: IO ()
main = getArgs >>= main_with_args

main_with_args :: [String] -> IO ()
main_with_args args =
    case args of
    ["--help"] -> usage
    ["-h"] -> usage
    ["--skel", filename] -> test_skeleton filename
    ["--skel"] -> test_skeleton_stdin
    ["--upd", filename] -> test_update filename
    ["--upd"] -> test_update_stdin
    [] -> putStrLn ("missing arguments") >> usage
    _ -> putStrLn ("invalid arguments: " ++ unwords args) >> usage

usage :: IO ()
usage =
    do putStrLn "usage: robber_tests <option> [filename]"
       putStrLn "  --skel     read and dump world skeleton"
       putStrLn "  --upd      read and dump world update"
       exitWith ExitSuccess

test_skeleton :: String -> IO ()
test_skeleton filename =
    do contents <- readFile filename
       parseTest p_world_skeleton contents

test_skeleton_stdin :: IO ()
test_skeleton_stdin = read_skeleton >>= print

test_update_stdin :: IO ()
test_update_stdin = read_update >>= print

test_update :: String -> IO ()
test_update filename =
    do contents <- readFile filename
       parseTest p_world_update contents

